package syncloud.dao.sqljet;

import org.tmatesoft.sqljet.core.SqlJetException;
import org.tmatesoft.sqljet.core.SqlJetTransactionMode;
import org.tmatesoft.sqljet.core.table.ISqlJetCursor;
import org.tmatesoft.sqljet.core.table.ISqlJetTable;
import org.tmatesoft.sqljet.core.table.SqlJetDb;
import syncloud.core.log.Logger;
import syncloud.dao.IStateDao;
import syncloud.dao.State;

import java.util.ArrayList;
import java.util.List;

public class StateDao implements IStateDao {
    private static Logger logger = Logger.getLogger(StateDao.class);
    private SqlJetDb db;

    public StateDao(String dbPath) {
        db = new Database(dbPath).open();
    }

    @Override
    public void insert(State state) {
        try {
            db.beginTransaction(SqlJetTransactionMode.WRITE);
            try {
                ISqlJetTable table = db.getTable(StateTable.TABLE_NAME);
                table.insert(getData(state));
            } finally {
                db.commit();
            }
        } catch (Throwable e) {
            logger.error("unable to insert: " + state, e);
        }
    }

    @Override
    public State get(String fullname) {
        try {
            db.beginTransaction(SqlJetTransactionMode.WRITE);
            try {
                ISqlJetTable table = db.getTable(StateTable.TABLE_NAME);
                ISqlJetCursor cursor = table.lookup(StateTable.NDX_FULL_NAME, fullname);
                if (!cursor.eof()) {
                    State state = read(cursor);
                    cursor.close();
                    return state;
                }
            } finally {
                db.commit();
            }
        } catch (Throwable e) {
            logger.error("unable to get " + fullname, e);
        }
        return null;
    }

    @Override
    public void delete(String fullname) {
        try {
            db.beginTransaction(SqlJetTransactionMode.WRITE);
            try {
                ISqlJetTable table = db.getTable(StateTable.TABLE_NAME);
                ISqlJetCursor cursor = table.lookup(StateTable.NDX_FULL_NAME, fullname);
                while (!cursor.eof()) {
                    cursor.delete();
                }
                cursor.close();
            } finally {
                db.commit();
            }
        } catch (Throwable e) {
            logger.error("unable to delete: " + fullname, e);
        }
    }

    @Override
    public void update(State state) {
        try {
            db.beginTransaction(SqlJetTransactionMode.WRITE);
            try {
                ISqlJetTable table = db.getTable(StateTable.TABLE_NAME);
                ISqlJetCursor cursor = table.lookup(StateTable.NDX_FULL_NAME, state.fullname);
                while (!cursor.eof()) {
                    cursor.update(getData(state));
                    cursor.next();
                }
                cursor.close();
            } finally {
                db.commit();
            }
        } catch (Throwable e) {
            logger.error("unable to update: " + state, e);
        }
    }

    private State read(ISqlJetCursor cursor) {
        try {
            String fullname = cursor.getString(StateTable.COL_FULL_NAME);
            String version = null;
            if (!cursor.isNull(StateTable.COL_VERSION))
                version = cursor.getString(StateTable.COL_VERSION);
            return new State(fullname, version);
        } catch (SqlJetException e) {
            logger.error("unable to read", e);
            return null;
        }
    }

    private Object[] getData(State state) {
        List<Object> data = new ArrayList<Object>();
        data.add(state.fullname);
        data.add(state.version);
        return data.toArray();
    }
}
